class Paddle extends GameEntity {
  PVector position = new PVector(0, 0);
  float width = 80, height = 12;
  AABB body = new AABB(0, 0, width, height);  
  
  void update(float deltaTime){
    //Make the paddle follow the mouse
    position.x = constrain(
      mouseX - state.arena.left - this.width/2, 
      1, 
      state.arena.width - this.width - 1
    );
    
    //Update the bounding box
    body.xMin = position.x;
    body.yMin = position.y;
    body.xMax = position.x + width;
    body.yMax = position.y + height;
  }
  
  void draw(){
    color paddleFill = color(220);//color(50, 240, 50);
    color paddleBorder = color(120);//color(0, 100, 0);
    
    //The central rectangle
    noStroke();
    fill(paddleFill);
    rect(position.x + this.height/2, position.y, this.width - this.height, this.height);
    
    //Rounded ends
    stroke(paddleBorder);
    strokeWeight(2);
    fill(paddleFill);
    arc(
      position.x + this.height/2, 
      position.y + this.height/2,
      this.height,
      this.height,
      PI / 2,
      TWO_PI - PI/2
    );
    
    arc(
      position.x + this.width - this.height/2, 
      position.y + this.height/2,
      this.height,
      this.height,
      -PI / 2,
      PI / 2
    );
    
    //Top and bottom border
    line(position.x + height/2, position.y, position.x+width-height/2, position.y);
    line(position.x + height/2, position.y+height, position.x+width-height/2, position.y+height);
    
    /*
    //Highlight
    strokeWeight(4);
    stroke(blendColor(paddleFill, color(50), ADD));
    line(position.x+height/2-2, position.y+4, position.x+width-height/2+2, position.y+4);
    */
    /*
    stroke(0, 100, 0);
    strokeWeight(1);
    fill(0, 200, 0);
    rect(position.x, position.y, this.width, this.height);
    //*/
  }
  
  Collision checkForCollision(PLine path, float radius){
    return intersectLineWithExpandedAABB(path, body, radius);
  };
  
  void onCollision(Collision collision, Ball ball){
    state.playSound("hitpaddle");
    
    //Change the ball's direction depending on where the it hit the paddle.
    //Only applies if the ball hit the top of the paddle.
    if (collision.surfaceNormal.y == -1){ 
      //Find out how far from the center of the paddle the ball bounced (-1..0..1).
      float bouncePosition = (collision.position.x - position.x - this.width/2) / ( this.width/2 );
      
      //Rotate the velocity vector slightly to the left (CCW) if the ball hits the left side 
      //of the paddle, and slightly to the right otherwise. If it hits the middle ("safe zone"),
      //the velocity is not altered.
      final float safeZone = 0.1;
      final float maxRotation = radians(45);   //Rotate the vector by -45 to 45 degrees
      final float maxReflection = radians(70); //The velocity direction must never be more than 70 deg away from "straight up"
      
      if ( abs(bouncePosition) > safeZone ){
        float angle = ((abs(bouncePosition) - safeZone)/(1 - safeZone)) * maxRotation * sign(bouncePosition);
        angle = constrain( 
          angle + sign(ball.velocity.x) * PVector.angleBetween(ball.velocity, collision.surfaceNormal),
          -maxReflection,
          maxReflection
        );
        ball.velocity.set( rotateVector(new PVector(0, -ball.velocity.mag()), angle) );     
        
        /*
        //This is an alternate approach that directly sets the reflection angle instead of 
        //modifying the default one.
        float angleOfReflection = map(bouncePosition, -1, 1, -60, 60);
        PVector newVelocity = rotateVector(collision.surfaceNormal, radians(angleOfReflection));
        newVelocity.mult(ball.velocity.mag());
        */
      }
    }
  };
  
  float sign(float num){
    return num >= 0 ? 1 : -1;
  }
}
